#!/usr/bin/speedy

use strict;
use Template;
use CGI qw(:standard);

use vars qw(%entity);

our($conn, %prefs, $template);

######################################################################
#
# Set up SpeedyCGI
#

use CGI::SpeedyCGI;
my $sp = CGI::SpeedyCGI->new;
$sp->setopt('timeout', 720);
$sp->setopt('MaxBackends', 1);
$sp->setopt('BufsizGet',  10485760);
$sp->setopt('BufsizPost', 10485760);
$sp->add_shutdown_handler(sub {
	opendir(DIR,"$main::prefs{'MailDir'}/tmp");
	while (defined (my $nuke = readdir DIR)) {
		next if $nuke =~ /^\..*$/;
		unlink "$main::prefs{'MailDir'}/tmp/$nuke";
	}
	closedir(DIR);
	$conn->disconnect() if $conn;
});

######################################################################
#
# Some stuff to initialize the WebMail client
#

#
# You don't need to give an OTP if your $ENV{'REMOTE_ADDR'} fits this regex
#
my $authenticated_host = '^192.168.2.[23]$';

# After 600 sec without activity a session will becoma invalid
our $session_timeout = 600;

$ENV{'HOME'} = "/home/httpd/htdocs/pronto/";

use lib "/home/httpd/htdocs/pronto";
use lib "/home/httpd/htdocs/pronto/web";
use lib "/usr/local/lib/pronto";
require "/usr/local/lib/pronto/prontolib.pl";

&read_prefs();
if (! defined $conn ) {
	$conn = &open_db_conn;
	# I assume that this is a new session, so clean up the tmp dir
	opendir(DIR,"$main::prefs{'MailDir'}/tmp");
	while (defined (my $nuke = readdir DIR)) {
		next if $nuke =~ /^\..*$/;
		unlink "$main::prefs{'MailDir'}/tmp/$nuke";
	}
	closedir(DIR);
	unlink "$main::prefs{'MailDir'}/prontoget.running";
}
if (! defined $template ) {
	$template = new Template({INCLUDE_PATH => "/home/httpd/htdocs/pronto/web/templates",
				  POST_CHOMP => 1});
}

######################################################################
#
# The actual Pronto WebMail client functions
#

#
# This isn't the login screen => the session has to be validated
#

if (param('t') ne "login") {
	&check_session();
}

#
# Handle the login screens
#

use Login;

if (param('t') eq "login") {
	my $authenticated = 0;
	if ($ENV{'REMOTE_ADDR'} =~ /$authenticated_host/) {
		$authenticated = 1;
	}
	if ( (param('userid')) && ((param('pass')) || ($authenticated == 1)) ) {
		&Login::login_third(param('userid'), $session_timeout, $authenticated, param('pass'));
	}
	elsif (param('userid')) {
		&Login::login_second(param('userid'));
	}
	else {
		&Login::login_first();
	}
}

#
# Create the standard frameset
#

if (param('t') eq "frames") {
	print header;
	$template->process("frames.html", {session => param('session')}) || die $template->error();
}

#
# The second frameset on the right side of the screen
#

if (param('t') eq "right_frame") {
	print header;
	$template->process("right_frame.html", {session => param('session')}) || die $template->error();
}

#
# The folder list
#

use Folders;
use Pronto::FolderTree;
use HTMLWidget::FolderTree;

if (param('t') eq "folders") {
	our $folder_tree = new HTMLWidget::FolderTree($template);
	our $normal = 1;
	our $bold = 2;

	&Folders::create(param('session'));

	sub set_win_title {
	}
}

#
# The message list
#

use Messages;
use HTMLWidget::FolderTree;

if (param('t') eq "messages") {
	our $folder_tree = new HTMLWidget::FolderTree;
	our (%ft_id, $message_clist);
	&Messages::create(param('session'), param('box'));

	sub set_labels {
		return;
	}
}

#
# View a selected message
#

use View;
use MIME::Parser;

if (param('t') eq "view") {
	our $parser;
	our $HTMLWIDGET = "GtkHTML";
	$parser = new MIME::Parser;

	&View::viewit(param('message'));
}

#
# Change message read status
#

if (param('t') eq "status") {
	my @messages = param('message');
	my $flag = param('flag');
	if (param('actionbox')) {
		@messages = param('actionbox'); 
		$flag     = param('helperval');
	}
	die("Message and/or Flag undefined") unless (@messages && $flag);	
	&status($flag, @messages);
	print redirect("/pronto/wpronto.pl?t=folders&session=".param('session'));
}

#
# Move message to another folder
#

if (param('t') eq "move") {
	my @messages = param('message');
	my $to = param('to');
	if (param('actionbox')) {
		@messages = param('actionbox'); 
		$to       = param('helperval');
	}
	die("Message and/or box undefined") unless (@messages && $to);
	&move($to, @messages);
	print redirect("/pronto/wpronto.pl?t=folders&session=".param('session'));
}

#
# Copy message to another folder
#

use Pronto::Data::Message;

if (param('t') eq "copy") {
	my @messages = param('message');
	my $to = param('to');
	if (param('actionbox')) {
		@messages = param('actionbox'); 
		$to       = param('helperval');
	}
	die("Message and/or box undefined") unless (@messages && $to);
	&Pronto::Data::Message::copy($to, @messages);
	print redirect("/pronto/wpronto.pl?t=folders&session=".param('session'));
}

#
# Delete, that is mark as read and move to deleted items, message
#

if (param('t') eq "delete") {
	my @messages = param('message');
	if (param('actionbox')) { @messages=param('actionbox'); }
	die("Message(s) undefined") unless (@messages);

	&status('read', @messages) if ($prefs{'DeleteMarksAsRead'} eq 'y');
	&move(4, @messages);
	print redirect("/pronto/wpronto.pl?t=folders&session=".param('session'));
}

#
# Delete message from database
#

if (param('t') eq "erase") {
	my @messages = param('erase');
	if (param('actionbox')) { @messages=param('actionbox'); }
	die("Message(s) undefined") unless (@messages);
	use lib "/home/httpd/htdocs/pronto/web";
	use Pronto::Data::Message;
	&Pronto::Data::Message::delete(@messages);
	print redirect("/pronto/wpronto.pl?t=messages&session=".param('session')."&box=4");
}

#
# Create search windows and do the query
#

use Search;

if (param('t') eq "search") {
	if (param('select')) {
		&Search::searchwin(param('session'));
	}
	else {
		our $message_clist;
		my $subject = param('subject' ) ? param('subject' ) : "_foo_";
		my $from    = param('from' ) ? param('from' ) : "_foo_";
		my $body    = param('body' ) ? param('body' ) : "_foo_";
		my $box     = param('box' ) ? param('box' ) : "_foo_";
		my $limit   = param('limit' ) ? param('limit' ) : "_foo_";
		&Search::process(param('session'), $subject, $from, $body, $box, $limit);
	}
}

#
# Everything that needs a compose window
#

use SendWin;

if (param('t') eq "send") {
	if (param('action') eq "compose") {
		&SendWin::compose(param('session'));
	}
	if (param('action') eq "send") {
		if (param('attached') ne "") {
			my $buffer;
			my $filename = $main::prefs{'MailDir'}."/tmp/".param('attached');
			open (OUTFILE,">>$filename");
			while (my $bytesread=read(param('attached'),$buffer,1024)) {
				print OUTFILE $buffer;
			}
		}

		my $refto  = param('refto' ) ? param('refto' ) : "foo";
		my $forrep = param('forrep') ? param('forrep') : "foo";
		my $cc     = param('cc'    ) ? param('cc'    ) : "foo";
		my $bcc    = param('bcc'   ) ? param('bcc'   ) : "foo";
		&SendWin::send($refto, $forrep, param('body'), param('to'), $cc, $bcc, param('subject'), param('acctid'), param('attached'));
	}
	if (param('action') eq "reply") {
		&SendWin::reply_forward("reply", param("message"), param('session'));
	}
	if (param('action') eq "forward") {
		&SendWin::reply_forward("forward", param("message"), param('session'));
	}
	sub send_out {
		&SendWin::send_out();
	}
}

#
# Check for new mail
#

use Getmail;

if (param('t') eq "getmail") {
	if (defined param('checkwin')) {
		&Getmail::checkwin(param('session'));
	}
	else {
		&Getmail::do_check(param('session'), param('account'));
	}
}

#
# Open the address book
#

if (param('t') eq "address") {
	&SendWin::address_win();
}

#
# Logout: Delete the session and exit.
#

if (param('t') eq "logout") {
	if (!param('session')) { html_die("No session given."); }
	
	my $sth = $conn->prepare("SELECT session from web_session WHERE session = " . param('session') . " AND valid_till > " . time()) || die $conn->errstr;
	$sth->execute() || die $conn->errstr;
	my ($session) = $sth->fetchrow();
	if (!defined $session) {
		html_die("Your session is no longer valid.");
	}
	
	$sth = $conn->prepare("DELETE FROM web_session WHERE session = ?");
	$sth->execute(param('session')) || die $conn->errstr;
	
	&Login::login_first();
	$sp->shutdown_now;
}

######################################################################
#
# Some additional required subs
#

# Checks the param('session') variable
sub check_session {
	my $valid_till = time() + $session_timeout;
	
	# Make sure that our session isn't expired
	if (!(param('session') =~ /^[0-9]+$/)) {
		&html_die("No session ID transmitted.");
	}
	my $sth = $conn->prepare("SELECT session,IP from web_session WHERE session = " . param('session') . " AND valid_till > " . time()) || die $conn->errstr;
	$sth->execute() || die $conn->errstr;
	my ($session, $ip) = $sth->fetchrow();
	if (!defined $session) {
		&html_die("Your session is no longer valid.");
	}
	elsif ($session != param('session')) {
		&html_die("Your session is no longer valid.");
	}
	elsif (!($ip eq $ENV{'REMOTE_ADDR'})) {
		&html_die("Your IP and the given session do not fit.");
	}
	
	# Update the timestamp that the user doesn't run out of time
	my $sql = "UPDATE web_session SET valid_till = " . (time()+$session_timeout) . " WHERE session = " . param('session') . " AND valid_till > " . time();
	my $affected_rows = $conn->do($sql);
	if (!defined $affected_rows) {
		die $sql . ": " . $conn->errstr;
	}
}

# Give some informative error messages
sub html_die {
	my $message = shift;
	
	eval { #we must not die in this function => recursion
	  print header, start_html(-BGCOLOR=>'#425C7D', "Error"), "<h1>An error occured</h1>\n<b>$message</b>", end_html;
	  exit 1;
	};
	if ($@) {
	  print STDERR $@;
	  exit 1;
	}
}

# Some kind of dummy function
sub get_win_size {
        my ($name,$dw,$dh) = @_;
        
        return ($dw,$dh) unless $prefs{'saveSize'} eq "y";
        return ($dw,$dh) unless defined $prefs{$name.'Width'} and
                            defined $prefs{$name.'Height'} >0;
        return ($prefs{$name.'Width'},
                $prefs{$name.'Height'});

}

# another dummy function
sub set_clist_drag_signals
{       
        return;
}

# This keeps the .pronto/tmp dir tidy, Pronto expects to find this sub here.
sub destroy_entity {
        my ($widget, $msgid) = @_;
        $entity{$msgid}->{'usecount'}--;
        if ($entity{$msgid}->{'usecount'} >= 1) {
                 return 1;
        } else {
                $entity{$msgid}->purge;
                delete $entity{$msgid};
                return 1;
        }
}

# My own simple implementation to change the read flag
sub status {
	my ($flag, @messages) = @_;
	my $sql;
	foreach my $message (@messages) {
		if ($flag eq "read") {
			$sql = "update messages set newmsg = 'n' where id = ?";
		} elsif ($flag eq "unread") {
			$sql = "update messages set newmsg = 'y' where id = ?";
		}
		my $query=$conn->prepare($sql);
		$query->execute($message) || die $conn->errstr;
	}
}

# A simple message move sub
sub move {
	my ($tobox, @messages) = @_;
	foreach my $message (@messages) {
		my $sql = "update messages set boxid = ? where id = ?";
		my $query = $conn->prepare($sql);
		$query->execute($tobox, $message) || die $conn->errstr;
	}
}
